@@ -10,19 +10,14 @@ from uvicorn.main import STARTUP_FAILURE
 from uvicorn.supervisors import ChangeReload, Multiprocess
 
 from . import globals
-from .config import Config
-
-os.environ['STATIC_DIRECTORY'] = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'static')
-os.environ['TEMPLATES_DIRECTORY'] = os.path.join(os.environ['STATIC_DIRECTORY'], 'templates')
 
 
-def run(self, *,
-        host: str = os.environ.get('HOST', '0.0.0.0'),
-        port: int = int(os.environ.get('PORT', '8080')),
+def run(*,
+        host: str = '0.0.0.0',
+        port: int = 8080,
         title: str = 'NiceGUI',
         favicon: Optional[str] = None,
         dark: Optional[bool] = False,
-        main_page_classes: str = 'q-pa-md column items-start gap-4',
         binding_refresh_interval: float = 0.1,
         show: bool = True,
         reload: bool = True,
@@ -31,20 +26,39 @@ def run(self, *,
         uvicorn_reload_includes: str = '*.py',
         uvicorn_reload_excludes: str = '.*, .py[cod], .sw.*, ~*',
         exclude: str = '',
-        ):
-    globals.config = Config(
-        host=host,
-        port=port,
-        title=title,
-        reload=reload,
-        favicon=favicon,
-        dark=dark,
-        main_page_classes=main_page_classes,
-        binding_refresh_interval=binding_refresh_interval,
-        excludes=[e.strip() for e in exclude.split(',')],
-    )
-    os.environ['HOST'] = globals.config.host
-    os.environ['PORT'] = str(globals.config.port)
+        ) -> None:
+    '''ui.run
+
+    You can call `ui.run()` with optional arguments:
+
+    :param host: start server with this host (default: `'0.0.0.0'`)
+    :param port: use this port (default: `8080`)
+    :param title: page title (default: `'NiceGUI'`, can be overwritten per page)
+    :param favicon: relative filepath to a favicon (default: `None`, NiceGUI icon will be used)
+    :param dark: whether to use Quasar's dark mode (default: `False`, use `None` for "auto" mode)
+    :param binding_refresh_interval: time between binding updates (default: `0.1` seconds, bigger is more CPU friendly)
+    :param show: automatically open the UI in a browser tab (default: `True`)
+    :param reload: automatically reload the UI on file changes (default: `True`)
+    :param uvicorn_logging_level: logging level for uvicorn server (default: `'warning'`)
+    :param uvicorn_reload_dirs: string with comma-separated list for directories to be monitored (default is current working directory only)
+    :param uvicorn_reload_includes: string with comma-separated list of glob-patterns which trigger reload on modification (default: `'.py'`)
+    :param uvicorn_reload_excludes: string with comma-separated list of glob-patterns which should be ignored for reload (default: `'.*, .py[cod], .sw.*, ~*'`)
+    :param exclude: comma-separated string to exclude elements (with corresponding JavaScript libraries) to save bandwidth
+      (possible entries: chart, colors, interactive_image, joystick, keyboard, log, scene, upload, table)
+
+    The environment variables `HOST` and `PORT` can also be used to configure NiceGUI.
+
+    To avoid the potentially costly import of Matplotlib, you set the environment variable `MATPLOTLIB=false`.
+    This will make `ui.plot` and `ui.line_plot` unavailable.
+    '''
+    globals.host = host
+    globals.port = port
+    globals.reload = reload
+    globals.title = title
+    globals.favicon = favicon
+    globals.dark = dark
+    globals.binding_refresh_interval = binding_refresh_interval
+    globals.excludes = [e.strip() for e in exclude.split(',')]
 
     if inspect.stack()[-2].filename.endswith('spawn.py'):
         return
@@ -53,7 +67,7 @@ def run(self, *,
         webbrowser.open(f'http://{host if host != "0.0.0.0" else "127.0.0.1"}:{port}/')
 
     def split_args(args: str) -> List[str]:
-        return args.split(',') if ',' in args else [args]
+        return [a.strip() for a in args.split(',')]
 
     # NOTE: The following lines are basically a copy of `uvicorn.run`, but keep a reference to the `server`.
 
@@ -61,7 +75,6 @@ def run(self, *,
         'nicegui:app' if reload else globals.app,
         host=host,
         port=port,
-        lifespan='on',
         reload=reload,
         reload_includes=split_args(uvicorn_reload_includes) if reload else None,
         reload_excludes=split_args(uvicorn_reload_excludes) if reload else None,
